'''
spamassassin module, version 0.2

(c) 2003 Jan ONDREJ (SAL) <ondrejj@salstar.sk>
                                                                                
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

'''

from avlib import *
import re,socket

__all__=['spamassassin','spamassassind']

def process(pf,spamstring):
    ret=[]
    exitstatus=''
    for line in pf.readlines():
      if re.search("^X-Spam-",line):
        ret.append(line)
        if re.search(spamstring,line):
          exitstatus='SPAM'
    return exitstatus,ret

class spamassassin(ascanner):
  '''
  SpamAssassin scanner
  
  Usage: spamassassin(['/usr/bin/spamassassin','-e'],SPAMSTRING,SA_MAX_FILE)
  
  Where: in [] is a path and argumets of spamassassin binary
         SPAMSTRING is a string, which define a spamassassin report,
           which is checked. If it is found, mail is assigned as 'SPAM'
           (by default '^X-Spam-Level: \*\*\*\*\*').
         SA_MAX_FILE is a number, which defines maximum file size,
           which can be tested. If a mail (with header) is larger,
           it is not checked (by default 500000).

  This scanner scans for spams. If a spam is returned, a string
  SPAM is returned as virus name. This scanner uses a spamassassin
  binary. For better performance, configure spamd and use
  spamassassind.
  
  WARNING! You need to install spamassassin into chroot manually.
  makechroot script does it not. If you want to use spamassasin
  with chroot and don't want to make a copy in chroot, use
  a spamassassind scanner, but remember, that scanning for spams
  are not done in chroot environment.
  '''
  is_spamscan=1
  name='SpamAssassin()'
  def __init__(self,arg,spamstring='^X-Spam-Level: \*\*\*\*\*',sa_max_file=500000):
      self.arg=arg
      self.SPAMSTRING=spamstring
      self.SA_MAX_FILE=sa_max_file
  def scanstream(self,stream):
      if len(stream)>self.SA_MAX_FILE:
        return '',[]
      pf=popen(self.arg)
      pf.write(stream)
      pf.close(1)
      pf.wait()
      return process(pf,self.SPAMSTRING)

class spamassassind(ascanner):
  '''
  SpamAssassinD scanner
  
  Usage: spamassassind(['localhost',783],SPAMSTRING,SA_MAX_FILE)

  Where: ['localhost',783] are the host and port of spamd.
         for SPAMSTRING and SA_MAX_FILE see spamassassin scanner description.
  
  This scanner scans for spams. If a spam is returned, a string
  SPAM is returned as virus name. This scanner uses a spamassassind
  daemon. If you are not familiar with it, you also can use
  spamassassin binary. Look at spamassassin scanner.
  '''
  is_spamscan=1
  name='SpamAssassind()'
  def __init__(self,arg,spamstring='^X-Spam-Level: \*\*\*\*\*',sa_max_file=500000):
      self.arg=arg
      self.SPAMSTRING=spamstring
      self.SA_MAX_FILE=sa_max_file
  def scanstream(self,stream):
      if len(stream)>self.SA_MAX_FILE:
        return '',[]
      s=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
      s.connect((self.arg[0],self.arg[1]))
      f=s.makefile('rw',BUFSIZE)
      s.send("PROCESS SPAMC/1.2\r\n")
      s.send("Content-length: "+str(len(stream))+"\r\n")
      s.send("User: test\r\n\r\n")
      s.send(stream)
      s.shutdown(1)
      s.close()
      return process(f,self.SPAMSTRING)
